from pandas.api.types import is_numeric_dtype
from pm4py.statistics.traces.generic.pandas import case_statistics
from pm4py.algo.filtering.pandas.variants import variants_filter
from pm4py.algo.filtering.pandas.timestamp import timestamp_filter
from pm4py.algo.filtering.pandas.start_activities import start_activities_filter
from pm4py.algo.filtering.pandas.end_activities import end_activities_filter
from pm4py.algo.filtering.pandas.cases import case_filter
from pm4py.algo.filtering.pandas.attributes import attributes_filter

def filter_workflow(workflow_df, nb_variants):
    variants_count = case_statistics.get_variant_statistics(workflow_df,
                                          parameters={case_statistics.Parameters.CASE_ID_KEY: 'case',
                                                      case_statistics.Parameters.ACTIVITY_KEY: 'activity',
                                                      case_statistics.Parameters.TIMESTAMP_KEY: 'timestamp'})
    variants_count = sorted(variants_count, key=lambda x: x['case'], reverse=True)
    top_variants = variants_count[:nb_variants]
    filtered_workflow = variants_filter.apply(workflow_df, [variant['variant'] for variant in top_variants],
                          parameters={variants_filter.Parameters.CASE_ID_KEY: "case",
                                      variants_filter.Parameters.ACTIVITY_KEY: "activity"})
    return filtered_workflow

def filter_timestamp(workflow, start, end, contained):
    if not contained:
        filtered_workflow = timestamp_filter.filter_traces_intersecting(workflow, start, end,
                  parameters={timestamp_filter.Parameters.CASE_ID_KEY: "case",
                              timestamp_filter.Parameters.TIMESTAMP_KEY: "timestamp"})
    else:
        filtered_workflow = timestamp_filter.filter_traces_contained(workflow, start, end,
                  parameters={timestamp_filter.Parameters.CASE_ID_KEY: "case",
                              timestamp_filter.Parameters.TIMESTAMP_KEY: "timestamp"})
    return filtered_workflow

def filter_start_step(workflow, start_steps):
    filtered_workflow = start_activities_filter.apply(workflow, start_steps,
                                              parameters={start_activities_filter.Parameters.CASE_ID_KEY: "case",
                                                          start_activities_filter.Parameters.ACTIVITY_KEY: "activity"})
    return filtered_workflow

def filter_end_step(workflow, end_steps):
    filtered_workflow = end_activities_filter.apply(workflow, end_steps,
                                              parameters={end_activities_filter.Parameters.CASE_ID_KEY: "case",
                                                          end_activities_filter.Parameters.ACTIVITY_KEY: "activity"})
    return filtered_workflow

def filter_case_performance(workflow, min_performance, max_performance, case_performance_scale):
    filtered_workflow = case_filter.filter_case_performance(workflow, min_case_performance=case_performance_scale[min_performance], 
                                                            max_case_performance=case_performance_scale[max_performance],
                                                            parameters={case_filter.Parameters.CASE_ID_KEY: "case",
                                                            case_filter.Parameters.TIMESTAMP_KEY: "timestamp"})
    return filtered_workflow

def filter_numeric_attributes(workflow, attribute_column, attribute_filter, attribute_scales):
    filtered_workflow = attributes_filter.apply_numeric(workflow, 
                                                        attribute_scales[attribute_column][int(attribute_filter[0])], 
                                                        attribute_scales[attribute_column][int(attribute_filter[1])],
                                             parameters={attributes_filter.Parameters.CASE_ID_KEY: "case",
                                                         attributes_filter.Parameters.ATTRIBUTE_KEY: attribute_column + ' Attribute'})
    return filtered_workflow

def filter_categorical_attributes(workflow, attribute_column, attribute_filter):
    filtered_workflow = attributes_filter.apply(workflow, attribute_filter,
                                          parameters={attributes_filter.Parameters.CASE_ID_KEY: "case", 
                                                      attributes_filter.Parameters.ATTRIBUTE_KEY: attribute_column + ' Attribute', 
                                                      attributes_filter.Parameters.POSITIVE: True})
    return filtered_workflow

def filter_attributes(workflow, attribute_column, attribute_filter, attribute_scales):
    if is_numeric_dtype(workflow[attribute_column + ' Attribute']):
        return filter_numeric_attributes(workflow, attribute_column, attribute_filter, attribute_scales)
    else:
        return filter_categorical_attributes(workflow, attribute_column, attribute_filter)

def filter_activities(workflow, excluded_activities):
    filtered_workflow = workflow[~workflow['activity'].isin(excluded_activities)]
    return filtered_workflow
 
def filter_time_before(workflow, time_before):
    filtered_workflow = workflow[workflow['diff_time']>=time_before]
    return filtered_workflow

def filter_self_transition(workflow):
    filtered_workflow = workflow[~(workflow['activity'] == workflow['previous_activity'])]
    return filtered_workflow